/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2018 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 5.04                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_RH850.c for Renesas RH850\F1H CPU.
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "device.h"  // Device specific header file, includes IAR SFR file

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings (configuration)
*/
#ifndef   OS_FSYS                    /* CPU Main clock frequency    */
  #define OS_FSYS 120000000u         /* Assume 120 MHz              */
#endif

#ifndef   OS_PCLK_TIMER              /* Peripheral clock for timer  */
  #define OS_PCLK_TIMER (80000000u)  /* May vary from CPU clock     */
#endif                               /* depending on CPU            */

#ifndef   OS_PCLK_UART               /* Peripheral clock for UART   */
  #define OS_PCLK_UART  (80000000u)  /* May vary from CPU clock     */
#endif                               /* depending on CPU            */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000u)
#endif

#define OS_TIMER_RELOAD  ((OS_PCLK_TIMER / OS_TICK_FREQ) - 1u)

/*********************************************************************
*
*       Configuration of communication to embOSView
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_DISABLED
#endif

/*********************************************************************
*
*       Local defines (sfrs and addresses used in RTOSInit.c)
*
**********************************************************************
*/

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/

#ifdef __cplusplus
extern "C" {
#endif
__interrupt void OS_ISR_Tick(void);
#ifdef __cplusplus
}
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value
*
*  Return value
*    Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return OSTM0CNT;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set
*
*  Return value
*    == 0; Interrupt pending flag not set
*    != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (ICOSTM0 & (1u << 12));
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_ISR_Tick()
*
* Function description
*   This is the code that gets called when the processor receives a
*   INTOSTM0 exception. INTOSTM0 is used as OS timer tick.
*/
#pragma vector = INTOSTM0_vector
__interrupt void OS_ISR_Tick(void) {
  OS_INT_EnterNestable();
  OS_INT_EnterIntStack();
  OS_TICK_Handle();
  OS_INT_LeaveIntStack();
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware (timer) required for embOS to run.
*    May be modified, if an other timer should be used.
*/
void OS_InitHW(void) {
  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_PCLK_TIMER, OS_TICK_FREQ, 0, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

  OS_INT_IncDI();
  //
  // Initialize timer for embOS: OSTM0, triggers INTOSTM0 execption
  //
  ICOSTM0  |= (1u <<  7);       // Initially disable INTOSTM0
  OSTM0TT   = (1u <<  0);       // Initially stop OSTM0
  OSTM0CMP  = OS_TIMER_RELOAD;  // Set reload value
  OSTM0CTL  = (0u <<  0)        // No interrupt on start
            | (0u <<  1);       // Select interval mode
  OSTM0EMU  = (0u <<  7);       // 0: stop during debug, 1: run during debug
  ICOSTM0   = (1u <<  0)        // Priority bit 0 (0b111 = 7 is lowest)
            | (1u <<  1)        // Priority bit 1 (0b111 = 7 is lowest)
            | (1u <<  2)        // Priority bit 2 (0x111 = 7 is lowest)
            | (1u <<  6)        // 0: use vector address, 1: use table reference
            | (0u <<  7)        // Re-enable INTOSTM0
            | (0u << 12)        // Initially clear interrupt request flag
            | (0u << 15);       // Select Edge detection
  OSTM0TS   = (1u <<  0);       // Re-Start OSTM0
  //
  // Setup values for usec precise system time functions
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize the optional communication for embOSView
  //
#if (OS_VIEW_IFSELECT != OS_VIEW_DISABLED)
  OS_COM_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This is basically the "core" of the idle loop.
*    This core loop can be changed, but:
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved. However, a simple program loop can be programmed
*    (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*  Function description
*    This routine is required for task-info via embOSView or high
*    resolution time measurement functions.
*    It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_TIME_GetTicks32();
  Cnt  = OS_TIMER_RELOAD - OSTM0CNT;
  //
  // Check if timer interrupt pending ...
  //
  if ((ICOSTM0 & (1u << 12))) {
    Cnt = OS_TIMER_RELOAD - OSTM0CNT;  // Interrupt pending, re-read timer and adjust result
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*  Function description
*    Convert Cycles into micro seconds.
*
*    If your clock frequency is not a multiple of 1 MHz,
*    you may have to modify this routine in order to get proper
*    diagnostics.
*
*    This routine is required for profiling or high resolution time
*    measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return (Cycles / (OS_TIMER_RELOAD / 1000u));
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Dummy routine.
*/
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
}

#else
  #error "Selected embOSView interface is currently not supported."
#endif

/*********************************************************************
*
*       __low_level_init()
*
*       Initialize clock generation and pll
*
*       Has to be modified, if another CPU clock frequency should be
*       used. This function is called during startup and
*       has to return 1 to perform segment initialization.
*
*       Because variables are not initialized before __low_level_init() is called,
*       no access or call which relies on any variable can be used here.
*/
unsigned char __low_level_init(int pe) {
  OS_USEPARA(pe);

  MOSCC=0x05;
  MOSCST=0xFFFF;
  protected_write(PROTCMD0,PROTS0,MOSCE,0x01u);
  while ((MOSCS&0x04u) != 0x04u);

  if((PLL0S&0x04u) != 0x04u)
  {
   PLL0C=0x0000021D;
   protected_write(PROTCMD1,PROTS1,PLL0E,0x01u);
   while((PLL0S&0x04u) != 0x04u);
  }
  if((PLL1S&0x04u) != 0x04u)
  {
   PLL1C=0x00000a27;
   protected_write(PROTCMD1,PROTS1,PLL1E,0x01u);
   while((PLL1S&0x04u) != 0x04u);
  }

  protected_write(PROTCMD1,PROTS1,CKSC_CPUCLKD_CTL,0x01u);
  while(CKSC_CPUCLKD_ACT!=0x01u);

  protected_write(PROTCMD1,PROTS1,CKSC_CPUCLKS_CTL,0x03u);
  while(CKSC_CPUCLKS_ACT!=0x03u);

  protected_write(PROTCMD1,PROTS1,CKSC_IPERI2S_CTL,0x02u);
  while(CKSC_IPERI2S_ACT!=0x02u);

  return 1;  // Always return 1 to enable segment initilization !
}

/****** End Of File *************************************************/
